"use strict";

exports.__esModule = true;
exports.createNodeId = createNodeId;
exports.decorateBulkObject = decorateBulkObject;
exports.getLastBuildTime = getLastBuildTime;
exports.getPluginStatus = getPluginStatus;
exports.isPriorityBuild = isPriorityBuild;
exports.isShopifyId = isShopifyId;
exports.parseImageExtension = parseImageExtension;
exports.parseShopifyId = parseShopifyId;
exports.processShopifyImages = processShopifyImages;
exports.setLastBuildTime = setLastBuildTime;

var _gatsbySourceFilesystem = require("gatsby-source-filesystem");

var _shopifyTypes = require("./shopify-types");

const pattern = /^gid:\/\/shopify\/(\w+)\/(.+)$/;

function createNodeId(shopifyId, gatsbyApi, {
  typePrefix
}) {
  return gatsbyApi.createNodeId(`${typePrefix}${shopifyId}`);
}

function getPluginStatus(gatsbyApi) {
  var _gatsbyApi$store$getS;

  return ((_gatsbyApi$store$getS = gatsbyApi.store.getState().status.plugins) === null || _gatsbyApi$store$getS === void 0 ? void 0 : _gatsbyApi$store$getS[`gatsby-source-shopify`]) || {};
}

function getLastBuildTime(gatsbyApi, pluginOptions) {
  const {
    typePrefix
  } = pluginOptions;
  const status = getPluginStatus(gatsbyApi);
  return status[`lastBuildTime${typePrefix}`] && new Date(status[`lastBuildTime${typePrefix}`]);
}

function setLastBuildTime(gatsbyApi, pluginOptions, currentBuildTime) {
  const {
    typePrefix
  } = pluginOptions;
  const status = getPluginStatus(gatsbyApi);
  gatsbyApi.actions.setPluginStatus({ ...status,
    [`lastBuildTime${typePrefix}`]: currentBuildTime
  });
}

function isPriorityBuild(pluginOptions) {
  const {
    CI,
    GATSBY_CLOUD,
    GATSBY_IS_PR_BUILD,
    NETLIFY,
    CONTEXT
  } = process.env;
  const isGatsbyCloudPriorityBuild = CI === `true` && GATSBY_CLOUD === `true` && GATSBY_IS_PR_BUILD !== `true`;
  const isNetlifyPriorityBuild = CI === `true` && NETLIFY === `true` && CONTEXT === `production`;
  return pluginOptions.prioritize !== undefined ? pluginOptions.prioritize : isGatsbyCloudPriorityBuild || isNetlifyPriorityBuild;
}

function isShopifyId(shopifyId) {
  return pattern.test(shopifyId);
}

function parseShopifyId(shopifyId) {
  return shopifyId.match(pattern) || [];
}

function decorateBulkObject(input) {
  if (input && typeof input === `object`) {
    if (Array.isArray(input)) {
      return input.map(decorateBulkObject);
    }

    const obj = { ...input
    };

    for (const key of Object.keys(obj)) {
      obj[key] = decorateBulkObject(obj[key]);
    } // We must convert ID to ShopifyID so that it doesn't collide with Gatsby's internal ID


    if (obj.id) {
      obj.shopifyId = obj.id;
      delete obj.id;
    }

    return obj;
  }

  return input;
}

async function processShopifyImages({
  actions: {
    createNode
  },
  createNodeId,
  cache
}, node) {
  const type = parseShopifyId(node.shopifyId)[1];
  const imageFields = _shopifyTypes.shopifyTypes[type].imageFields;

  if (imageFields) {
    for (const fieldPath of imageFields) {
      const image = fieldPath.split(`.`).reduce((acc, value) => acc[value], node);

      if (image && parseImageExtension(image.originalSrc) !== `gif`) {
        const fileNode = await (0, _gatsbySourceFilesystem.createRemoteFileNode)({
          url: image.originalSrc,
          cache,
          createNode,
          createNodeId,
          parentNodeId: node.id
        });
        image.localFile___NODE = fileNode.id;
      }
    }
  }
}

function parseImageExtension(url) {
  const basename = url.split(`?`)[0];
  const dot = basename.lastIndexOf(`.`);

  if (dot !== -1) {
    return basename.slice(dot + 1);
  } else {
    throw new Error(`Could not parse file extension from Shopify image URL: ${url}`);
  }
}
//# sourceMappingURL=helpers.js.map