"use strict";

exports.__esModule = true;
exports.makeResolveGatsbyImageData = makeResolveGatsbyImageData;

var _fs = require("fs");

var _fetchRemoteFile = require("gatsby-core-utils/fetch-remote-file");

var _gatsbyPluginImage = require("gatsby-plugin-image");

var _getShopifyImage = require("./get-shopify-image");

var _helpers = require("./helpers");

async function getImageBase64({
  imageAddress,
  directory,
  contentDigest
}) {
  // Downloads file to the site cache and returns the file path for the given image (this is a path on the host system, not a URL)
  const filePath = await (0, _fetchRemoteFile.fetchRemoteFile)({
    url: imageAddress,
    directory,
    cacheKey: contentDigest
  });
  const buffer = (0, _fs.readFileSync)(filePath);
  return buffer.toString(`base64`);
}
/**
 * Download and generate a low-resolution placeholder
 *
 * @param lowResImageFile
 */


function getBase64DataURI({
  imageBase64
}) {
  return `data:image/png;base64,${imageBase64}`;
}

function makeResolveGatsbyImageData(cache) {
  return async function resolveGatsbyImageData(image, {
    formats = [`auto`],
    layout = `constrained`,
    ...remainingOptions
  }) {
    // Sharp cannot optimize GIFs so we must return null in that case
    const ext = (0, _helpers.parseImageExtension)(image.originalSrc);
    if (ext === `gif`) return null;
    let placeholderURL;

    const generateImageSource = (filename, width, height, toFormat) => {
      return {
        width,
        height,
        placeholder: ``,
        format: toFormat,
        src: (0, _getShopifyImage.urlBuilder)({
          width,
          height,
          baseUrl: filename,
          format: toFormat,
          options: {}
        })
      };
    };

    const sourceMetadata = {
      width: image.width,
      height: image.height,
      format: ext
    };

    if (remainingOptions && remainingOptions.placeholder === `blurred`) {
      // This function returns the URL for a 20px-wide image, to use as a blurred placeholder
      const lowResImageURL = (0, _gatsbyPluginImage.getLowResolutionImageURL)({ ...remainingOptions,
        formats,
        layout,
        sourceMetadata,
        pluginName: `gatsby-source-shopify`,
        filename: image.originalSrc,
        generateImageSource
      });
      const imageBase64 = await getImageBase64({
        imageAddress: lowResImageURL,
        directory: cache.directory,
        contentDigest: image.internal.contentDigest
      });
      placeholderURL = getBase64DataURI({
        imageBase64
      });
    }

    return (0, _gatsbyPluginImage.generateImageData)({ ...remainingOptions,
      placeholderURL,
      formats,
      layout,
      sourceMetadata,
      pluginName: `gatsby-source-shopify`,
      filename: image.originalSrc,
      generateImageSource
    });
  };
}
//# sourceMappingURL=resolve-gatsby-image-data.js.map