"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.makeSourceFromOperation = makeSourceFromOperation;

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _readline = require("readline");

var _shiftLeft = require("shift-left");

var _helpers = require("./helpers");

var _processBulkResults = require("./process-bulk-results");

var _errors = require("./errors");

function makeSourceFromOperation(finishLastOperation, completedOperation, cancelOperationInProgress, gatsbyApi, pluginOptions, lastBuildTime) {
  return async function sourceFromOperation(op) {
    const {
      reporter
    } = gatsbyApi;
    const operationTimer = reporter.activityTimer(`source ${lastBuildTime ? `changed ` : ``}shopify ${op.name}`);
    operationTimer.start();

    try {
      if ((0, _helpers.isPriorityBuild)(pluginOptions)) {
        await cancelOperationInProgress();
      } else {
        await finishLastOperation();
      }

      const {
        bulkOperationRunQuery: {
          userErrors,
          bulkOperation
        }
      } = await op.execute();

      if (userErrors.length) {
        reporter.panic(userErrors.map(e => {
          const msg = e.field ? `${e.field.join(`.`)}: ${e.message}` : e.message;
          return {
            id: _errors.pluginErrorCodes.bulkOperationFailed,
            context: {
              sourceMessage: `Couldn't initiate bulk operation query`
            },
            error: new Error(msg)
          };
        }));
      }

      const resp = await completedOperation(bulkOperation.id);

      if (parseInt(resp.node.objectCount, 10) === 0) {
        operationTimer.end();
        return;
      }

      const {
        body: jsonLines
      } = await (0, _nodeFetch.default)(resp.node.url);
      const rl = (0, _readline.createInterface)({
        input: jsonLines,
        crlfDelay: Infinity
      });
      const results = [];

      for await (const line of rl) {
        results.push(JSON.parse(line));
      }

      const nodeCount = await (0, _processBulkResults.processBulkResults)(gatsbyApi, pluginOptions, results);
      operationTimer.setStatus(`${nodeCount} nodes`);
      operationTimer.end();
    } catch (e) {
      if (e instanceof _errors.OperationError) {
        const code = _errors.pluginErrorCodes.bulkOperationFailed;

        if (e.node.status === `CANCELED`) {
          if ((0, _helpers.isPriorityBuild)(pluginOptions)) {
            /**
             * There are at least two production sites for this Shopify
             * store trying to run an operation at the same time.
             */
            reporter.panic({
              id: _errors.pluginErrorCodes.apiConflict,
              error: e,
              context: {}
            });
          } else {
            // A prod build canceled me, wait and try again
            operationTimer.setStatus(`This operation has been canceled by a higher priority build. It will retry shortly.`);
            operationTimer.end();
            await new Promise(resolve => setTimeout(resolve, 5000));
            await sourceFromOperation(op);
          }
        }

        if (e.node.errorCode === `ACCESS_DENIED`) {
          reporter.panic({
            id: code,
            context: {
              sourceMessage: `Your credentials don't have access to a resource you requested`
            },
            error: e
          });
        }

        reporter.panic({
          id: _errors.pluginErrorCodes.unknownSourcingFailure,
          context: {
            sourceMessage: (0, _shiftLeft.shiftLeft)`
              Operation ${op.name} failed after ${e.node.objectCount} objects
              - With status: ${e.node.status} - error code: ${e.node.errorCode}
            `
          },
          error: e
        });
      }

      if (e instanceof _errors.HttpError) {
        reporter.panic({
          id: _errors.pluginErrorCodes.unknownApiError,
          context: {
            sourceMessage: `Received error ${e.response.status} from Shopify: ${await e.response.text()}`
          },
          error: e
        });
      }

      console.log(e);
      reporter.panic({
        id: _errors.pluginErrorCodes.unknownSourcingFailure,
        context: {
          sourceMessage: `Could not source from bulk operation`
        },
        error: e
      });
    }
  };
}
//# sourceMappingURL=source-from-operation.js.map